#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 29 14:14:44 2022

@author: Ariel L. Morrison
"""

def makePredictions(timeFrame,var,numYears,testNum):
    datadir = '/Users/arielmor/Desktop/SAI/data/ARISE/data/model_output'
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    import tensorflow as tf
    
    from preprocessingData import preprocessDataForPrediction 
    model = tf.keras.models.load_model('pfrost_down_to_bedrock_logistic_regression_predict_'+str(var)+'_'+str(timeFrame))
    
    lat,lon,features_train,features_val,features_test,\
        labels_train,labels_val,labels_test,X_train,y_train,\
            X_val,y_val,X_test,y_test,lenTime,testMemNum = preprocessDataForPrediction(str(
                                                    timeFrame),str(var),numYears)
    
    ## ------ Predictions! ------ ##
    y_pred_test  = model.predict(X_test)
    ## -------------------------- ## 
    
    ## ------ Prediction confidence ------ ##
    '''
        y_pred_test shape is two columns
        1st column = probability that map comes from CONTROL
        2nd column = probability that map comes from FEEDBACK
        Length of array = 2 x lenTime
        20 years of training data = 40 predictions --> 1st 20 = control, 2nd 20 = feedback
    '''
    y_pred_CONTROL= np.round(y_pred_test[:lenTime],decimals=3)
    y_pred_FEEDBACK = np.round(y_pred_test[lenTime:],decimals=3)
    np.save(datadir + '/y_pred_CONTROL_' + str(testMemNum) + '_' + str(var) + '_' + str(timeFrame) + '.npy',
            y_pred_CONTROL)
    np.save(datadir + '/y_pred_FEEDBACK_' + str(testMemNum) + '_' + str(var) + '_' + str(timeFrame) + '.npy',
            y_pred_FEEDBACK)
    
    
    ens = ['[2.]','[3.]','[4.]','[5.]','[6.]','[7.]','[8.]','[9.]','[10.]']
    fig, ax = plt.subplots(figsize=(8,4),dpi=1200)
    for ensNum in range(len(ens)):
        y_pred_CONTROL = np.load(datadir + '/y_pred_CONTROL_' + str(ens[ensNum]) + '_' + str(var) + '_' + str(timeFrame) + '.npy')
        y_pred_FEEDBACK = np.load(datadir + '/y_pred_FEEDBACK_' + str(ens[ensNum]) + '_' + str(var) + '_' + str(timeFrame) + '.npy')
        
        ## - add markers to signify if prediction is right or wrong - ##
        df = pd.DataFrame.from_dict(dict(controlprediction=(1-y_pred_CONTROL[:,0]),
                                         feedbackprediction=y_pred_FEEDBACK[:,0]))
        df.index.name="index"
        df.reset_index(inplace=True)
        df["criteria"] = df.controlprediction > 0.5
        df["criteria2"] = df.feedbackprediction > 0.5
    
        ## - make figure - ##
        ax = df.controlprediction.plot(c='xkcd:dull pink',linewidth=0.7,label='SSP2-4.5',alpha=0.8)
        df[df.criteria].plot(kind='scatter',x='index',y='controlprediction',
                                     ax=ax,c='xkcd:dull pink',marker='o',alpha=0.8,s=7)
        df[~df.criteria].plot(kind='scatter',x='index',y='controlprediction',
                                      ax=ax,c='xkcd:dull pink',marker='x',alpha=0.8,s=7)
        df.feedbackprediction.plot(c='xkcd:powder blue',ax=ax,linewidth=0.7,label='ARISE-SAI-1.5',alpha=0.8)
        df[df.criteria2].plot(kind='scatter',x='index',y='feedbackprediction',
                                      ax=ax,c='xkcd:powder blue',marker='o',alpha=0.8,s=7)
        df[~df.criteria2].plot(kind='scatter',x='index',y='feedbackprediction',
                                       ax=ax,c='xkcd:powder blue',marker='x',alpha=0.8,s=7)
    del df
    
    ## testing member in bold
    y_pred_CONTROL = np.load(datadir + '/y_pred_CONTROL_[1.]_' + str(var) + '_' + str(timeFrame) + '.npy')
    y_pred_FEEDBACK = np.load(datadir + '/y_pred_FEEDBACK_[1.]_' + str(var) + '_' + str(timeFrame) + '.npy')
    
    ## - add markers to signify if prediction is right or wrong - ##
    df = pd.DataFrame.from_dict(dict(controlprediction=(1-y_pred_CONTROL[:,0]),
                                     feedbackprediction=y_pred_FEEDBACK[:,0]))
    df.index.name="index"
    df.reset_index(inplace=True)
    df["criteria"] = df.controlprediction > 0.5
    df["criteria2"] = df.feedbackprediction > 0.5

    ## - make figure - ##
    ax = df.controlprediction.plot(c='r',linewidth=2,label='SSP2-4.5')
    df[df.criteria].plot(kind='scatter',x='index',y='controlprediction',
                                 ax=ax,c='r',marker='o',s=15)
    df[~df.criteria].plot(kind='scatter',x='index',y='controlprediction',
                                  ax=ax,c='r',marker='x',s=15)
    df.feedbackprediction.plot(c='xkcd:cobalt blue',ax=ax,linewidth=2,label='ARISE-SAI-1.5')
    df[df.criteria2].plot(kind='scatter',x='index',y='feedbackprediction',
                                  ax=ax,c='xkcd:cobalt blue',marker='o',s=15)
    df[~df.criteria2].plot(kind='scatter',x='index',y='feedbackprediction',
                                   ax=ax,c='xkcd:cobalt blue',marker='x',s=15)
    
    from matplotlib.lines import Line2D
    custom_lines = [Line2D([0], [0], color='red', lw=2),
                Line2D([0], [0], color='xkcd:cobalt blue', lw=2)]
    
    plt.legend(custom_lines, ['SSP2-4.5', 'ARISE-SAI-1.5'])

    # handles, labels = plt.gca().get_legend_handles_labels()
    # labels, ids = np.unique(labels, return_index=True)
    # handles = [handles[i] for i in ids]
    # leg = plt.legend(handles, labels, loc='lower right', fancybox=True)
    # for legobj in leg.legendHandles: legobj.set_linewidth(2)
    
    if timeFrame == 'winter' or timeFrame == 'non_growing_season':
        plt.xlim([0,19])
        ax.set_xticks([0,2,4,6,8,10,12,14,16,18])
        ax.set_xticklabels(['2035','2037','2039','2041','2043','2045','2047','2049','2051','2053'])
    elif timeFrame == 'monthly':
        plt.xlim([0,lenTime])
    else:
        plt.xlim([0,34])
        ax.set_xticks([0,5,10,15,20,25,30,34])
        ax.set_xticklabels(['2035','2040','2045','2050','2055','2060','2065','2069'])
    plt.axhline(0.5,0,lenTime,linestyle='dashed',color='k',linewidth=0.6)
    ax.set_yticks([])
    plt.ylim([0,1])
    ax.set(xlabel=None)
    plt.ylabel('incorrect                        correct',
               fontweight='bold')
    ax.fill_between(range(lenTime),0,0.5,alpha=0.18,color='k')
    if var == 'ALT': plt.title('a) Model prediction confidence, active layer depth', fontsize=12, fontweight='bold')
    elif var == 'TSOI': plt.title('b) Model prediction confidence, soil temperature', fontsize=12, fontweight='bold')
    plt.savefig('/Users/arielmor/Desktop/SAI/data/ARISE/figures/all_ensemble_members_prediction_confidence_'\
                +str(timeFrame)+'_'+str(var)+'.jpg',bbox_inches='tight',dpi=1200)
    plt.show()
    
    
    
    ## ------------------------------------ ##
    #### Contribution map
    from plottingFunctions import get_colormap, make_maps
    model = tf.keras.models.load_model('pfrost_down_to_bedrock_logistic_regression_predict_'+str(var)+'_'+str(timeFrame))
    lenLat = 37; lenLon = 288; 
    brbg_cmap,rdbu_cmap,jet,magma,reds,hot,seismic = get_colormap(21)
    ens = ['001', '002', '003', '004', '005', '006', '007', '008', '009', '010']
    
    mapWeights = (model.layers[0].get_weights()[0].reshape(lenLat,lenLon))
    
    if var == 'ALT':
        from tippingPoints import findPermafrost
        ariseANN,arise,altmaxAnnFEEDBACK,lat,lon = findPermafrost(
            'b.e21.BW.f09_g17.SSP245-TSMLT-GAUSS-DEFAULT.', '203501-206912')
    elif var == 'TSOI':
        import xarray as xr
        ds = xr.open_dataset(
            '/Users/arielmor/Desktop/SAI/data/ARISE/data/b.e21.BW.f09_g17.SSP245-TSMLT-GAUSS-DEFAULT.001.clm2.h0.ALT.203501-206912_NH.nc')
        lat = ds.lat[41:]
        lon = ds.lon
        ds.close()
        import pickle
        with open('/Users/arielmor/Desktop/SAI/data/ARISE/data/permafrost_temperature_FEEDBACK.pkl', 'rb') as fp:
            ariseANN = pickle.load(fp)
        
        
    if timeFrame == 'annual':
        if var == 'TSOI':
            # ariseANN[str('00') + str(testNum)] = ariseANN[str('00') + str(testNum)] - 273.15
            mapTitle = 'b) Permafrost temperature contributions'
            contribution = mapWeights*(np.nanmean(ariseANN[str('00') + str(testNum)][:,:-7,:,],axis=0))
        elif var == 'ALT' or var == 'ALTMAX':
            mapTitle = 'a) Active layer depth contributions'
            contribution = mapWeights*(np.nanmean(ariseANN[ens[0]][:,:-7,:,]*100.,axis=0))
     

    fig,ax = make_maps(contribution,lat[:-7],lon,
                        -4,4,21,rdbu_cmap,
                        'ARISE-SAI-1.5 input x weights',str(mapTitle),
                        'weights_'+str(timeFrame)+'_'+str(var),'both',False,False)
    
    
    
    ## ----------------------------------- ##
    ## ------ Accuracy metrics ------ ##    
    ''' Accuracy of each prediction vs label '''
    acc_CONTROL = np.asarray(np.asarray(np.equal(np.round(np.squeeze(y_pred_CONTROL)),
                                                 labels_test[0,:lenTime]),dtype='int32'),dtype='float')
    acc_FEEDBACK = np.asarray(np.asarray(np.equal(np.round(np.squeeze(y_pred_FEEDBACK)),
                                                  labels_test[1,:lenTime]),dtype='int32'),dtype='float')
    
    print("-------------------------------------------")
    print("Correct predictions (label = 1; above 50% confidence)")
    print("CONTROL", acc_CONTROL)
    print("FEEDBACK", acc_FEEDBACK)
    print("-------------------------------------------")
    
    fig, (ax1, ax2) = plt.subplots(2, figsize=(8,4), dpi=900)
    ax2.scatter(range(lenTime),acc_FEEDBACK,color='b',label='ARISE-SAI')
    ax2.legend(loc="lower right", fancybox=True, fontsize=11)
    ax1.scatter(range(lenTime),acc_CONTROL,color='r',label='SSP2-4.5')
    ax1.legend(loc="lower right", fancybox=True, fontsize=11)
    if timeFrame == 'winter' or timeFrame == 'non_growing_season':
        ax1.set_xticks([0,2,4,6,8,10,12,14,16,18])
        ax1.set_xticklabels([])
        ax2.set_xticks([0,2,4,6,8,10,12,14,16,18])
        ax2.set_xticklabels(['2035','2037','2039','2041','2043','2045','2047','2049','2051','2053'])
    else:
        ax1.set_xticks([0,2,4,6,8,10,12,14,16,18])
        ax1.set_xticklabels([])
        ax2.set_xticks([0,2,4,6,8,10,12,14,16,18])
        ax2.set_xticklabels(['2035','2037','2039','2041','2043','2045','2047','2049','2051','2053'])
    ax2.set_yticks([0,1]) 
    ax2.set_yticklabels(['False','True'], fontweight='bold')
    ax1.set_yticks([0,1]) 
    ax1.set_yticklabels(['False','True'], fontweight='bold')
    plt.savefig('/Users/arielmor/Desktop/SAI/data/ARISE/figures/prediction_accuracy_'\
                +str(timeFrame)+'_'+str(var)+'_UPDATED_092723.jpg', bbox_inches='tight',dpi=900); plt.show()
    ## ------------------------------ ##
   
    '''
    First number is likelihood of map coming from first column of testing data,
    second number is likelihood of second column 
    
    First 20 numbers = control, second 20 = arise
    '''
    ## ------ Map of weights ------ ##
    from plottingFunctions import get_colormap, make_maps
    lenLat = 49; lenLon = 288; 
    brbg_cmap,rdbu_cmap,jet,magma,reds,hot,seismic = get_colormap(21)
    
    mapWeights = (model.layers[0].get_weights()[0].reshape(lenLat,lenLon))
    
    if var == 'ER':
        vmins = -0.016; vmaxs = 0.016
    else:
        vmins = -0.04; vmaxs = 0.04
    fig,ax = make_maps(mapWeights,lat[30:-6],lon,
                        vmins,vmaxs,21,rdbu_cmap,
                        'weights','weights for '+str(timeFrame),
                        'weights_'+str(timeFrame)+'_'+str(var),'both')
    ## ----------------------------- ##
    return